local _,stash = ...

--[[
    Any local variables pointing to StashSettings before PLAYER_LOGIN will not follow to the
    new StashSettings when savedvariables are loaded, necessitating the need to either listen
    for PLAYER_LOGIN to assign a local variable, or directly use StashSettings. To get around
    this, use an empty metatable with setters/getters to the actual savedvar.

    local _,stash = ...
    local settings = stash.settings

    Now use settings.variable as usual:
        settings.LockWindow = not settings.LockWindow
]]

StashSettings = {} -- actual savedvar

-- default settings; note that a nil default setting is discouraged since that means "undefined"; use false instead
local defaults = {
    -- settings panel
    LockWindow = false, -- lock window size and position
    PerCharacterPosition = false, -- remember size and position per character
    NoESCape = false, -- don't dismiss window with ESC key
    ShowMinimapButton = false, -- use minimap button to summon window
    ShowRarityBorders = false, -- show rarity borders
    HideMoneyTooltip = false, -- hide tooltip for gold per character
    DefaultSearchTypes = false, -- defaults filters to search types too
    DefaultCurrentRealm = false, -- defaults filters to current realm/faction
    AutoFocus = false, -- grab searchbox focus when window shown
    TabFocus = false, -- grab searchbox focus when Tab key hit
    ResetOnClose = false, -- reset filters when window dismissed
    DontConfirmReset = false, -- don't confirm to reset characters
    AddToTooltips = false, -- don't add stash details to gametooltip
    -- filter panel
    SearchTypesToo = false, -- include types and subtypes in searches
    CurrentRealmOnly = false, -- filters results to current realm/faction
    SortByRarity = false, -- sorts items by rarity
    FilterRealm = false, -- dropdown filter for realm
    FilterCharacter = false, -- dropdown filter for character
    FilterLocation = false, -- dropdown filter for location
    FilterType = false, -- dropdown filter for type
    FilterSubType = false, -- dropdown filter for subtype
    FilterEquipSlot = false, -- dropdown filter for equipslot
    -- hidden settings
    IsFilterPanelOpen = false, -- not in settings; whether the itemsPanel pullout is open
}

-- metatable must remain empty for this to reliably work; which it will because setter never rawsets
local function getter(self,key)
    if StashSettings[key]==nil then -- if actual setting undefined, save a default value if one exists
        StashSettings[key] = defaults[key]
    end
    return StashSettings[key]
end

-- setter never does a rawset so table remains empty and __index always fires
local function setter(self,key,value)
    StashSettings[key] = value -- saving to savedvar instead
end

stash.settings = setmetatable({},{__index = getter, __newindex = setter})
